/**
 *  '$RCSfile: AccessControlForSingleFile.java,v $'
 *    Purpose: A Class that loads eml-access.xml file containing ACL 
 *             for a metadata document into relational DB
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Jivka Bojilova
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2003/01/21 20:10:10 $'
 * '$Revision: 1.2 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

import java.io.*;
import java.sql.*;
import java.util.Stack;
import java.util.Vector;
import java.util.Hashtable;
import java.net.URL;
import java.net.MalformedURLException;


/** 
 * A Class that loads eml-access.xml file containing ACL for a metadata
 * document into relational DB. It extends DefaultHandler class to handle
 * SAX parsing events when processing the XML stream.
 */
public class AccessControlForSingleFile implements AccessControlInterface 
{

 
 
  private String docId;
  private String principal;
  private int    permission;
  private String permType;
  private String permOrder;


 
  /**
   * Construct an instance of the AccessControlForSingleFile class.
   * @param myAccessNumber  the docid or docid with dev will be controlled
   * @param myprincipal  the principal will have permission
   * @param myPermission the permission will be given
   * @param myPermType  the permsission type, allow or deny
   * @param myPermOrder  the permission order, allowFirst or denyFirst
   */
  public AccessControlForSingleFile(String myAccessionNumber,
                                    String myPrincipalName,
                                    String myPermission,
                                    String myPermType,
                                    String myPermOrder) 
                                   throws Exception
  {
    try
    {
      //Get rid of dev if myaccessNumber has one;
      docId = MetaCatUtil.getDocIdFromString(myAccessionNumber);
      if (docId == null || docId.equals(""))
      {
        throw new Exception("Docid couldn't be null");
      }
      // Check principal
      principal = myPrincipalName;
      if (principal == null || principal.equals(""))
      {
        throw new Exception("principal couldn't be null");
      }
      // get permission value
      permission = AccessControlList.intValue(myPermission);
      if (permission == -1)
      {
        throw new Exception("permission "+ myPermission + " is not valid");
      }
      // check permission type
      permType = myPermType;
      // if permtype is not allow or deny, throw a exception
      if (permType == null
          || (!permType.equals(AccessControlInterface.ALLOW)
          && !permType.equals(AccessControlInterface.DENY)))
      {
        throw new Exception("permtype should be "+AccessControlInterface.ALLOW+
                            " or " +AccessControlInterface.DENY);
      }
      // check permission order
      permOrder = myPermOrder;
      //if permission order is not allowFirst or denyFirst, assing it dneyFirst
      if (permOrder == null
          || !permOrder.equals("AccessControlInterface.DENYFIRST"))
      {
        permOrder = AccessControlInterface.ALLOWFIRST;
      }
      //debugMessage
      MetaCatUtil.debugMessage("docid in AccessControlForSingleFiel: " +
                                docId, 30);
      MetaCatUtil.debugMessage("principal in AccessControlForSingleFiel: " +
                                principal, 30);
      MetaCatUtil.debugMessage("permission in AccessControlForSingleFiel: " +
                                permission, 30);
      MetaCatUtil.debugMessage("permType in AccessControlForSingleFiel: " +
                                permType, 30);
      MetaCatUtil.debugMessage("permOrder in AccessControlForSingleFiel: " +
                                permOrder, 30);
    }
    catch (Exception e)
    {
      MetaCatUtil.debugMessage("Error in construct of AccessControlForSingle" +
                               "File: " + e.getMessage(), 30);
      throw e;
    }
  }
  

  


 

  /**
   * Method to insert records into xml_access table
   */
  
  public void insertPermissions() throws SQLException 
  {
    PreparedStatement pstmt = null;
    DBConnection conn = null;
    int serialNumber = -1;
    try
    {
      //check out DBConnection
      conn=DBConnectionPool.getDBConnection
                               ("AccessControlForSingleFiel.insertPermissions");
      serialNumber=conn.getCheckOutSerialNumber();
      pstmt = conn.prepareStatement(
              "INSERT INTO xml_access " + 
              "(docid, principal_name, permission, perm_type, perm_order " +
              ") VALUES (?,?,?,?,?)");
     
      // Bind the values to the query
      pstmt.setString(1, docId);
      pstmt.setString(2, principal);
      pstmt.setInt(3, permission);
      pstmt.setString(4, permType);
      pstmt.setString(5, permOrder);
      //pstmt.setString(6, AccessControlInterface.ACLID);
      pstmt.execute();
    }//try
    catch (SQLException e)
    {
      MetaCatUtil.debugMessage("Error in AccessControlForSingleFile.insert" +
                               "Permissions: " + e.getMessage(), 30);
      throw e;
    }
    finally
    {
      try
      {
        pstmt.close();
      }
      finally
      {
        DBConnectionPool.returnDBConnection(conn, serialNumber);
      }
    }
   
  }


}
