/**
 *  '$RCSfile: DBConnectionPool.java,v $'
 *    Purpose: A class represent a DBConnection pool. Another user can use the
 *    object to initial a connection pool, get db connection or return it. 
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Jing Tao
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2003/07/28 03:50:45 $'
 * '$Revision: 1.12 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

import java.io.*;
import java.util.Vector;
import java.lang.*;
import java.sql.*;
import java.util.Stack;
import java.util.Hashtable;
import java.util.Enumeration;

/** 
 * A class represent a DBConnection pool. Another user can use the
 * object to initial a connection pool, get db connection or return it.
 * This a singleton class, this means only one instance of this class could
 * be in the program at one time. 
 */
public class DBConnectionPool implements Runnable
{

  //static attributes
  private static DBConnectionPool instance;
  private static Vector connectionPool;
  private static Thread runner;
  
  //maximum connection number in the connection pool
  final static int MAXIMUMCONNECTIONNUMBER=
                  Integer.parseInt(MetaCatUtil.getOption("maximumConnections"));
  
  //inintila connection number int the connection pool
  final static int INITIALCONNECTIONNUMBER=
                  Integer.parseInt(MetaCatUtil.getOption("initialConnections"));
 
  //the number to increase connection pool size
  final static int INCREASECONNECTIONNUMBER=
                Integer.parseInt(MetaCatUtil.getOption("incrementConnections"));
                
  //maximum age for a connection (in milli seconds)
  final static long MAXIMUMAGE =
                  Long.parseLong(MetaCatUtil.getOption("maximumConnectionAge"));
  //maximum connection time for a connection ( in milli second)
  final static long MAXIMUMCONNECTIONTIME =
                 Long.parseLong(MetaCatUtil.getOption("maximumConnectionTime"));
  //maximum number for using a connection.
  final static int MAXIMUMUSAGENUMBER =
                  Integer.parseInt(MetaCatUtil.getOption("maximumUsageNumber"));
  //the parameter if run dbconncestionrecyclethread or not
  final static String DBCONNECTIONRECYCLETHREAD =
                          MetaCatUtil.getOption("runDBConnectionRecycleThread");
  //the cycle time of connection recycle action
  final static long CYCLETIMEOFDBCONNECTION = 
               Long.parseLong(MetaCatUtil.getOption("cycleTimeOfDBConnection"));
               
  //the number for trying to check out a connection in the pool in 
  //getDBConnection method
  final static int LIMIT = 2;
  
  final static int FREE = 0; //status of a connection
  final static int BUSY = 1; //statis of a connection
  /**
   * Returns the single instance, creating one if it's the
   * first time this method is called.
   */
  public static synchronized DBConnectionPool getInstance()
                                 throws SQLException 
  {
    MetaCatUtil.debugMessage
                      ("MaximumConnectionNumber: "+MAXIMUMCONNECTIONNUMBER, 49);
    MetaCatUtil.debugMessage
                     ("Intial connection number: "+INITIALCONNECTIONNUMBER, 49);
    MetaCatUtil.debugMessage
                 ("Increated connection Number: "+INCREASECONNECTIONNUMBER, 49);
    MetaCatUtil.debugMessage
                  ("Maximum connection age: "+MAXIMUMAGE, 49);
    MetaCatUtil.debugMessage
                  ("Maximum connection time: "+MAXIMUMCONNECTIONTIME, 49);
    MetaCatUtil.debugMessage
                  ("Maximum usage count: "+MAXIMUMUSAGENUMBER, 49);
    MetaCatUtil.debugMessage
             ("Running recycle thread or not: "+DBCONNECTIONRECYCLETHREAD, 49);
    MetaCatUtil.debugMessage
             ("Cycle time of recycle: "+CYCLETIMEOFDBCONNECTION, 49); 
    if (instance == null) 
    {
      instance = new DBConnectionPool();
    }
    return instance;
  }//getInstance


  /**
   * This is a private constructor since it is singleton
   */
   
  private DBConnectionPool()  throws SQLException 
  {
    connectionPool = new Vector();
    initialDBConnectionPool();
    //running the thread to recycle DBConnection
    if (DBCONNECTIONRECYCLETHREAD.equals("on"))
    {
      runner = new Thread(this);
      runner.start();
    }
  }//DBConnection

  /**
   * Method to get the size of DBConnectionPool
   */
  public int getSizeOfDBConnectionPool()
  {
    return connectionPool.size();
  }
  
  
  /**
   * Method to initial a pool of DBConnection objects 
   */
  private void initialDBConnectionPool() throws SQLException 
  {

    DBConnection dbConn = null;
    
    for ( int i = 0; i < INITIALCONNECTIONNUMBER; i++ ) 
    {
      //create a new object of DBConnection
      //this DBConnection object has a new connection in it
      //it automatically generate the createtime and tag
      dbConn = new DBConnection();
      //put DBConnection into vetor
      connectionPool.add(dbConn);
    }    
    
  
  }//initialDBConnectionPool
  
  /**
   * Method to get Connection object (Not DBConnection)
   */
  /*public static Connection getConnection() throws SQLException
  {
    DBConnection dbConn = null;
    //get a DBConnection
    dbConn = getDBConnection();
    //get connection object in DBConnection object
    //The following getConnections method is in DBConnection class
    return dbConn.getConnections();
  }*/
  
 
  /**
   * Method to get a DBConnection in connection pool
   * 1) try to get a DBConnection from DBConnection pool
   * 2) if 1) failed, then check the size of pool. If the size reach the
   *    maximum number of connection, throw a exception: couldn't get one
   * 3) If the size is less than the maximum number of connectio, create some
   *    new connections and recursive get one
   * @param methodName, the name of method which will check connection out
   */
  public static synchronized DBConnection getDBConnection(String methodName) 
                                                throws SQLException
  {
    DBConnection db = null;
    int random = 0; //random number
    int index = 0; //index
    int size = 0; //size of connection pool
    MetaCatUtil.debugMessage("Try to checking out connection...", 65);
    size = connectionPool.size();
    MetaCatUtil.debugMessage("size of connection pool: "+size, 55);
    
     //try every DBConnection in the pool
    //every DBConnection will be try LIMITE times
    for (int j=0 ; j<LIMIT; j++)
    {
       //create a random number as the started index for connection pool
      //So that the connection ofindex of 0 wouldn't be a the heaviest user
      random = (new Double (Math.random()*100)).intValue();
      for (int i=0; i<size; i++)
      {
        index =(i+random)%size;
        db = (DBConnection) connectionPool.elementAt(index);
        MetaCatUtil.debugMessage("Index: "+index, 65);
        MetaCatUtil.debugMessage("Tag: "+db.getTag(), 65);
        MetaCatUtil.debugMessage("Status: "+db.getStatus(), 65);
        //check if the connection is free
        if (db.getStatus()==FREE)
        {
          //If this connection is good, return this DBConnection
          if (validateDBConnection(db))
          {
            
            //set this DBConnection status
            db.setStatus(BUSY);
            //increase checkout serial number
            db.increaseCheckOutSerialNumber(1);
            //increase one usageCount
            db.increaseUsageCount(1);
            //set method name to DBConnection
            db.setCheckOutMethodName(methodName);
            db.setAutoCommit(true);
            //debug message
            MetaCatUtil.debugMessage("The connection is checked out: "
                                       +db.getTag(), 65);
            MetaCatUtil.debugMessage("The method for checking is: " 
                                              +db.getCheckOutMethodName(), 65);
            MetaCatUtil.debugMessage("The age is "+db.getAge(), 65);
            MetaCatUtil.debugMessage("The usage is "+db.getUsageCount(), 65);
            MetaCatUtil.debugMessage("The conection time it has: "
                                                +db.getConnectionTime(), 65);
            //set check out time
            db.setCheckOutTime(System.currentTimeMillis());
            //check it out
            return db;
          }//if
          else//The DBConnection has some problem
          {
            //close this DBConnection
            db.close();
            //remove it form connection pool
            connectionPool.remove(index);
            //insert a new DBConnection to same palace
            db = new DBConnection();
            connectionPool.insertElementAt(db, index);
          }//else
        }//if
      }//for
    }//for
    
    //if couldn't get a connection, we should increase DBConnection pool
    //if the connection pool size is less than maximum connection number
   
    if ( size < MAXIMUMCONNECTIONNUMBER )
    {
       if ((size+INCREASECONNECTIONNUMBER) < MAXIMUMCONNECTIONNUMBER)
       { 
         //if we can create INCREASECONNECTIONNUMBER of new DBConnection
         //add to connection pool
         for ( int i=0; i<INCREASECONNECTIONNUMBER; i++)
         {
           DBConnection dbConn = new DBConnection();
           connectionPool.add(dbConn);
         }//for
       }//if
       else
       {
         //There is no enough room to increase INCREASECONNECTIONNUMBER 
         //we create new DBCoonection to Maximum connection number
         for (int i= size+1; i<= MAXIMUMCONNECTIONNUMBER; i++)
         {
           DBConnection dbConn = new DBConnection();
           connectionPool.add(dbConn);
         }//for
       }//else
   
    }//if
    else
    {
      /*throw new SQLException("The maximum of " +MAXIMUMCONNECTIONNUMBER + 
                            " open db connections is reached." +
                            " New db connection to MetaCat" +
                            " cannot be established.");*/
       MetaCatUtil.debugMessage("The maximum of " +MAXIMUMCONNECTIONNUMBER + 
                            " open db connections is reached." +
                            " New db connection to MetaCat" +
                            " cannot be established.", 1);
       return null;
       //if couldn't get a connection, sleep 20 seconds and try again.
       /*try
       {
         System.out.println("sleep");
         Thread.sleep(2000);
       }
       catch (Exception e)
       {
       }*/
         
      
    }//else
    
    //recursive to get new connection    
    return getDBConnection(methodName); 
  }//getDBConnection

 
  
  
  /** 
   * Method to check if a db connection works fine or not
   * Check points include:
   * 1. check the usageCount if it is too many
   * 2. check the dbconne age if it is too old
   * 3. check the connection time if it is too long
   * 4. run simple sql query
   *
   * @param dbConn, the DBConnection object need to check
   */
  private static boolean validateDBConnection (DBConnection dbConn)
  {
    
    
    //Check if the DBConnection usageCount if it is too many
    if (dbConn.getUsageCount() >= MAXIMUMUSAGENUMBER )
    {
      MetaCatUtil.debugMessage("Connection usageCount is too many: "+
      dbConn.getUsageCount(), 65);
      return false;
    }
    
    //Check if the DBConnection has too much connection time
    if (dbConn.getConnectionTime() >= MAXIMUMCONNECTIONTIME)
    {
      MetaCatUtil.debugMessage("Connection has too much connection time: "+
      dbConn.getConnectionTime(), 65);
      return false;
    }
    
    //Check if the DBConnection is too old
    if (dbConn.getAge() >=MAXIMUMAGE)
    {
      MetaCatUtil.debugMessage("Connection is too old: "+dbConn.getAge(), 65);
      return false;
    }
    
    //Try to run a simple query
    try
    {
      long startTime=System.currentTimeMillis();
      DatabaseMetaData metaData = dbConn.getMetaData();
      long stopTime=System.currentTimeMillis();
      //increase one usagecount
      dbConn.increaseUsageCount(1);
      //increase connection time
      dbConn.setConnectionTime(stopTime-startTime);
  
    }
    catch (Exception e)
    {
      MetaCatUtil.debugMessage("Error in validateDBConnection: "
                                +e.getMessage(), 30);
      return false;
    }
    
    return true;
    
  }//validateDBConnection()
  
  /**
   * Method to return a connection to DBConnection pool.
   * @param conn, the Connection object need to check in
   */
  public static synchronized void returnDBConnection(DBConnection conn, 
                                                              int serialNumber)
  {
    int index = -1;
    DBConnection dbConn = null;
  
    index = getIndexOfPoolForConnection(conn);
    if ( index ==-1 )
    {
      MetaCatUtil.debugMessage("Couldn't find a DBConnection in the pool"
                                  +" which have same tag to the returned"
                                  +" DBConnetion object", 30);
      return;
                                  
    }//if
    else
    {
      //check the paramter - serialNumber which will be keep in calling method
      //if it is as same as the object's checkoutserial number.
      //if it is same return it. If it is not same, maybe the connection already
      // was returned ealier.
      MetaCatUtil.debugMessage("serial number in Connection: "
                                      +conn.getCheckOutSerialNumber(), 65);
      MetaCatUtil.debugMessage("serial number in local: "+serialNumber, 65);
      if (conn.getCheckOutSerialNumber() == serialNumber)
      {
        dbConn = (DBConnection) connectionPool.elementAt(index);
        //set status to free
        dbConn.setStatus(FREE);
        //count connection time
        dbConn.setConnectionTime
                          (System.currentTimeMillis()-dbConn.getCheckOutTime());
                          
        //set check out time to 0
        dbConn.setCheckOutTime(0);
        
        MetaCatUtil.debugMessage("Connection: "+dbConn.getTag()+" checked in.",
                                                                        65);
        MetaCatUtil.debugMessage("Connection: "+dbConn.getTag()+"'s status: "
                                                    +dbConn.getStatus(), 65);
                                                                       
      }//if
      else
      {
        MetaCatUtil.debugMessage("This DBConnection couldn't return", 30);
      }//else
    }//else
   
 
  }//returnConnection
  
  /**
   * Given a returned DBConnection, try to find the index of DBConnection object
   * in dbConnection pool by comparing DBConnection' tag and conn.toString.
   * If couldn't find , -1 will be returned.
   * @param conn, the connection need to be found
   */
  private static synchronized int getIndexOfPoolForConnection(DBConnection conn)
  {
    int index = -1;
    String info = null;
    //if conn is null return -1 too
    if (conn==null)
    {
      return -1;
    }
    //get tag of this returned DBConnection
    info = conn.getTag();
    //if the tag is null or empty, -1 will be returned
    if (info==null || info.equals(""))
    {
      return index;
    }
    //compare this info to the tag of every DBConnection in the pool
    for ( int i=0; i< connectionPool.size(); i++)
    {
      DBConnection dbConn = (DBConnection) connectionPool.elementAt(i);
      if (info.equals(dbConn.getTag()))
      {
        index = i;
        break;
      }//if
    }//for
    
    return index;
  }//getIndexOfPoolForConnection  
  
  /**
   * Method to shut down all connections
   */
  public static void release()
  {
    
    //shut down the backgroud recycle thread
    if (DBCONNECTIONRECYCLETHREAD.equals("on"))
    {
      runner.interrupt();
    }
    //cose every dbconnection in the pool
    synchronized(connectionPool)
    {
      for (int i=0;i<connectionPool.size();i++)
      {
        try
        {
          DBConnection dbConn= (DBConnection) connectionPool.elementAt(i);
          dbConn.close();
        }//try
        catch (SQLException e)
        {
          MetaCatUtil.debugMessage("Error in release connection: "
                                            +e.getMessage(), 10);
        }//catch
      }//for
    }//synchronized
  }//release()
  
  /**
   * periodically to recycle the connection
   */
  public void run()
  {
    DBConnection dbConn = null;
    //keep the thread running
    while (true)
    {
      //check every dbconnection in the pool
      synchronized(connectionPool)
      {
        for (int i=0; i<connectionPool.size(); i++)
        {
          dbConn = (DBConnection) connectionPool.elementAt(i);
          
          //if a DBConnection conncectioning time for one check out is greater 
          //than 30000 milliseconds print it out
          if (dbConn.getStatus()==BUSY && 
            (System.currentTimeMillis()-dbConn.getCheckOutTime())>=30000)
          {
            MetaCatUtil.debugMessage("This DBConnection is checked out for: "
            +(System.currentTimeMillis()-dbConn.getCheckOutTime())/1000
            +" secs" , 1);
            MetaCatUtil.debugMessage(dbConn.getTag(), 1);
            MetaCatUtil.debugMessage("method: "
                                          +dbConn.getCheckOutMethodName(), 10);
            
          }
          
          //check the validation of free connection in the pool
          if (dbConn.getStatus() == FREE)
          {
            try
            {
              //try to print out the warning message for every connection
              if (dbConn.getWarningMessage()!=null)
              {
                MetaCatUtil.debugMessage("Warning for connection "
                  +dbConn.getTag()+" : "+ dbConn.getWarningMessage(), 10);
              }
              //check if it is valiate, if not create new one and replace old one
              if (!validateDBConnection(dbConn))
              {
                MetaCatUtil.debugMessage("Recyle it: "+ dbConn.getTag(), 65);
                //close this DBConnection
                dbConn.close();
                //remove it form connection pool
                connectionPool.remove(i);
                //insert a new DBConnection to same palace
                dbConn = new DBConnection();
                connectionPool.insertElementAt(dbConn, i);
               }//if
            }//try
            catch (SQLException e)
            {
              MetaCatUtil.debugMessage("Error in DBConnectionPool.run: "
                                              +e.getMessage(), 20);
            }//catch
          }//if
        }//for
      }//synchronize   
      //Thread sleep 
      try
      {
        Thread.sleep(CYCLETIMEOFDBCONNECTION);
      }
      catch (Exception e)
      {
        MetaCatUtil.debugMessage("Error in DBConnectionPool.run: "
                                              +e.getMessage(), 20);
      }
    }//while
  }//run
  
  /**
   * Method to get the number of free DBConnection in DBConnection pool
   */
  public static synchronized int getFreeDBConnectionNumber()
  {
    int numberOfFreeDBConnetion = 0; //return number
    DBConnection db = null; //single DBconnection
    int poolSize = 0; //size of connection pool
    //get the size of DBConnection pool
    poolSize = connectionPool.size();
    //Check every DBConnection in the pool
    for ( int i=0; i<poolSize; i++)
    {
      
      db = (DBConnection) connectionPool.elementAt(i);
      //check the status of db. If it is free, count it
      if (db.getStatus() == FREE)
      {
        numberOfFreeDBConnetion++;
      }//if
    }//for
    //return the count result
    return numberOfFreeDBConnetion;
  }//getFreeDBConnectionNumber
      
   /**
   * Method to print out the method name which have busy DBconnection
   */
  public void printMethodNameHavingBusyDBConnection()
  {
    
    DBConnection db = null; //single DBconnection
    int poolSize = 0; //size of connection pool
    //get the size of DBConnection pool
    poolSize = connectionPool.size();
    //Check every DBConnection in the pool
    for ( int i=0; i<poolSize; i++)
    {
      
      db = (DBConnection) connectionPool.elementAt(i);
      //check the status of db. If it is free, count it
      if (db.getStatus() == BUSY)
      {
        MetaCatUtil.debugMessage("This method having a busy DBConnection: "
                                    +db.getCheckOutMethodName(), 30);
        MetaCatUtil.debugMessage("The busy DBConnection tag is: "
                                    +db.getTag(), 30);
      }//if
    }//for
  
  }//printMethodNameHavingBusyDBConnection
  
  /**
   * Method to decrease dbconnection pool size when all dbconnections are idle
   * If all connections are free and connection pool size greater than 
   * initial value, shrink connection pool size to intital value
   */
  public static synchronized boolean shrinkConnectionPoolSize() 
  {
     int connectionPoolSize = 0; //store the number of dbconnection pool size
     int freeConnectionSize = 0; //store the number of free dbconnection in pool
     int difference = 0; // store the difference number between connection size
                         // and free connection
     boolean hasException = false; //to check if has a exception happend
     boolean result = false; //result
     DBConnection conn = null; // the dbconnection
     connectionPoolSize = connectionPool.size();
     freeConnectionSize = getFreeDBConnectionNumber();
     MetaCatUtil.debugMessage("Connection pool size: " +connectionPoolSize, 3);
     MetaCatUtil.debugMessage("Free Connection number: "+freeConnectionSize, 3);
     difference = connectionPoolSize - freeConnectionSize;
     
     //If all connections are free and connection pool size greater than 
     //initial value, shrink connection pool size to intital value
     if (difference == 0 && connectionPoolSize > INITIALCONNECTIONNUMBER)
     {
       //db connection having index from  to connectionpoolsize -1
       //intialConnectionnumber should be close and remove from pool
       for ( int i=connectionPoolSize-1; i >= INITIALCONNECTIONNUMBER ; i--)
       {
        
         //get the dbconnection from pool
         conn = (DBConnection) connectionPool.elementAt(i);
         
         try
         {
           //close conn
           conn.close();
         }//try
         catch (SQLException e)
         { 
           // set hadException ture
           hasException = true;
           MetaCatUtil.debugMessage("Couldn't close a DBConnection in " +
                            "DBConnectionPool.shrinkDBConnectionPoolSize: " +
                            e.getMessage(), 30);
         }//catch
                                        
        //remove it from pool
        connectionPool.remove(i);
        // becuase enter the loop, set result true
        result = true;
       }//for
     }//if
     
     //if hasException is true ( there at least once exception happend)
     // the result should be false
     if (hasException)
     {
       result =false;
     }//if
     // return result
     return result;
  }//shrinkDBConnectionPoolSize
   
    /**
   * Method to decrease dbconnection pool size when all dbconnections are idle
   * If all connections are free and connection pool size greater than 
   * initial value, shrink connection pool size to intital value
   */
  public static synchronized void shrinkDBConnectionPoolSize() 
  {
     int connectionPoolSize = 0; //store the number of dbconnection pool size
     int freeConnectionSize = 0; //store the number of free dbconnection in pool
     int difference = 0; // store the difference number between connection size
                         // and free connection
    
     DBConnection conn = null; // the dbconnection
     connectionPoolSize = connectionPool.size();
     freeConnectionSize = getFreeDBConnectionNumber();
     MetaCatUtil.debugMessage("Connection pool size: " +connectionPoolSize, 35);
     MetaCatUtil.debugMessage("Free Connection number: "+freeConnectionSize, 35);
     difference = connectionPoolSize - freeConnectionSize;
     
     //If all connections are free and connection pool size greater than 
     //initial value, shrink connection pool size to intital value
     if (difference == 0 && connectionPoolSize > INITIALCONNECTIONNUMBER)
     {
       //db connection having index from  to connectionpoolsize -1
       //intialConnectionnumber should be close and remove from pool
       for ( int i=connectionPoolSize-1; i >= INITIALCONNECTIONNUMBER ; i--)
       {
        
         //get the dbconnection from pool
         conn = (DBConnection) connectionPool.elementAt(i);
         //make sure again the DBConnection status is free
         if (conn.getStatus()==FREE)
         {
           try
           {
             //close conn
             conn.close();
           }//try
           catch (SQLException e)
           { 
          
             MetaCatUtil.debugMessage("Couldn't close a DBConnection in " +
                            "DBConnectionPool.shrinkDBConnectionPoolSize: " +
                            e.getMessage(), 30);
           }//catch
        
           //remove it from pool
           connectionPool.remove(i);
         }//if
       
       }//for
     }//if
     
    
  }//shrinkDBConnectionPoolSize
   
  
}//DBConnectionPool
