/**
 *   '$Author: tao $'
 *     '$Date: 2003/04/26 22:19:22 $'
 * '$Revision: 1.26 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
package edu.ucsb.nceas.metacat;

import edu.ucsb.nceas.morpho.datapackage.Triple;
import edu.ucsb.nceas.morpho.datapackage.TripleCollection;

import java.io.StringReader;
import java.sql.*;
import java.util.*;
import java.net.*;


public class RelationHandler //implements Runnable
{
  private DBConnection connection = null;
  private String docid = null;
  private String docType = null;
  
  TripleCollection tripleForPackage = null;
   
  /** 
   * Constructor for this class.  finds all of the relations to a single xml
   * document and writes them to the database.  This includes transitive
   * relations.
   * @param docid the ID of the XML document to index.
   * @param doctype the doctype of this document
   * @param conn the db connection
   * @param list the triple list
   */
  public RelationHandler(String docid, String doctype, 
                        DBConnection conn, TripleCollection list)
              throws McdbException, SQLException, AccessionNumberException
  {
    this.connection = conn;
    this.docid = docid;
    this.docType = doctype;
    tripleForPackage = list;
    createRelations();
  }
  
   /**
   * insert the relations specified in the triples into xml_relation table
   */ 
  private void createRelations() 
              throws McdbException, SQLException, AccessionNumberException
  {
    String packagetype = docType;
    String subject = null;
    String subjectParentId = null;
    String subDoctype = null;
    String relationship = null;
    String relationshipParentId = null;
    String object = null;
    String objDoctype = null;
    PreparedStatement tstmt = null; // to insert each relation into xml_relation
   
    MetaCatUtil.debugMessage("Running relation handler!", 40);
   
    // first delete the relations for this package document if any
    deleteRelations(docid);
 
    //get the vetor of triples 
    Vector tripleList= new Vector();
    //get vector form tripleCollection
    if (tripleForPackage != null)
    {
      tripleList =tripleForPackage.getCollection();
    }
    
    if (tripleList != null && tripleList.size()>0) 
    {
      
       tstmt = connection.prepareStatement("INSERT INTO xml_relation (" +
                                    "docid,packagetype,subject,subdoctype," +
                                    "relationship, object, objdoctype) " + 
                                    "VALUES (?, ?, ?, ?, ?, ?, ?)");
      
      // go through tripe list 
      for (int i= 0; i<tripleList.size(); i++)
      {
        //increase usage count
        connection.increaseUsageCount(1);
        // Get the triple
        Triple triple = (Triple)tripleList.elementAt(i);
        MetaCatUtil.debugMessage("Info from triple: ", 30);
        MetaCatUtil.debugMessage("subject from triple:"+triple.getSubject(), 30);
        MetaCatUtil.debugMessage("relationship from triple:"+triple.getRelationship(),30);
        MetaCatUtil.debugMessage("object from triple: "+triple.getObject(),30);
        //subject = (new DocumentIdentifier(triple.getSubject())).getIdentifier();
        subject = MetaCatUtil.getDocIdFromString(triple.getSubject());
        relationship = triple.getRelationship();
        //object = (new DocumentIdentifier(triple.getObject())).getIdentifier();
        object = MetaCatUtil.getDocIdFromString(triple.getObject());
        
        if (subject != null && relationship != null && object != null)
        {
        
          //put the new relation into xml_relation
          MetaCatUtil.debugMessage("Insert into xml_relation table", 30);
          tstmt.setString(1, docid);
          MetaCatUtil.debugMessage("Insert docid into xml_relation table" + 
                                docid, 30);
          tstmt.setString(2, packagetype);
          tstmt.setString(3, subject);
          MetaCatUtil.debugMessage("Insert subject into xml_relation table" + 
                               subject, 30);
          tstmt.setString(4, subDoctype);
          tstmt.setString(5, relationship);
          MetaCatUtil.debugMessage("Insert relationship into xml_relation table" + 
                                relationship, 30);
          tstmt.setString(6, object);
          MetaCatUtil.debugMessage("Insert object into xml_relation table" + 
                                object, 30);
          tstmt.setString(7, objDoctype);
          tstmt.execute();  
        }//if
     
      }//for
    }//if
    
    if ( tstmt != null ) 
    {
      tstmt.close();
    }
  
   
  }
 
  /**
   * Deletes all of the relations with a docid of 'docid'.
   * @param docid the docid of the package which relations to delete.
   */
  public void deleteRelations(String docid) throws SQLException
  {
    try {
      PreparedStatement pstmt = connection.prepareStatement(
                                "DELETE FROM xml_relation " +
                                "WHERE docid = '" + docid + "'");
      //increase usage count
      connection.increaseUsageCount(1);
      pstmt.execute();
      pstmt.close();
    } catch(SQLException e) {
      MetaCatUtil.debugMessage("error in RelationHandler.deleteRelations(): " + 
                          e.getMessage(), 30);
      
      throw e;
    }
  }

  /**
   * Get the access file id for a package
   * @param docid the document identifier of the package
   * @return the document identifier of the access file for that package
   */
  public static String getAccessFileID(String docid) throws SQLException
  {
    String aclid = null;
    PreparedStatement pstmt = null;
    DBConnection dbConn = null;
    int serialNumber = -1;
    
    StringBuffer sql = new StringBuffer();
    sql.append("SELECT docid FROM xml_documents WHERE docid in (SELECT subject ");
    sql.append("FROM xml_relation WHERE docid='").append(docid);
    sql.append("' AND (");
    Vector accessdoctypes = MetaCatUtil.getOptionList(
                              MetaCatUtil.getOption("accessdoctype"));
    for(int i=0; i<accessdoctypes.size(); i++)
    {
      String atype = (String)accessdoctypes.elementAt(i);
      sql.append("doctype='").append(atype).append("'");
      if(i < accessdoctypes.size()-1)
      {
        sql.append(" OR ");
      }
    }
    sql.append("))");
    //System.out.println("new sql script: " + sql.toString());
    
    try
    {
      dbConn=DBConnectionPool.
                  getDBConnection("RelationHandler.getAccessFileID");
      serialNumber=dbConn.getCheckOutSerialNumber();
      pstmt = dbConn.prepareStatement(sql.toString());
      pstmt.execute();
      ResultSet rs = pstmt.getResultSet();
      boolean hasRow = rs.next();
      if (hasRow) {
        aclid = rs.getString(1);
      }
      pstmt.close();
    }//try
    finally
    {
      try
      {
        pstmt.close();
      }//try
      finally
      {
        DBConnectionPool.returnDBConnection(dbConn, serialNumber);
      }//finally
    }//finally
     
    MetaCatUtil.debugMessage("The access docid get from xml_relation is: "+
                              aclid, 30);
    return aclid;
  }

}
